from flask import Flask, render_template, Response, request, redirect, url_for, session
import cv2
import numpy as np
import rclpy
from rclpy.node import Node
from sensor_msgs.msg import Image
from cv_bridge import CvBridge
import threading

app = Flask(__name__, template_folder='/home/parkseongho/IJ3_ros/src/security_alert/security_alert/templates')
app.secret_key = 'your_secret_key'

# Global variables for storing the latest frames from ROS camera and notebook camera
latest_ros_frame = None
latest_notebook_frame = None

class ROSSubscriber(Node):
    def __init__(self):
        super().__init__('ros_subscriber')
        self.bridge = CvBridge()
        self.image = None
        self.subscription = self.create_subscription(
            Image,
            'camera/image',
            self.image_callback,
            10
        )
    
    def image_callback(self, msg):
        global latest_ros_frame
        try:
            self.image = self.bridge.imgmsg_to_cv2(msg, "bgr8")
            ret, jpeg = cv2.imencode('.jpg', self.image)
            if ret:
                latest_ros_frame = jpeg.tobytes()
        except Exception as e:
            print(f"Error in image callback: {e}")
            self.image = None

def generate_ros_stream():
    global latest_ros_frame
    while True:
        if latest_ros_frame:
            yield (b'--frame\r\n'
                   b'Content-Type: image/jpeg\r\n\r\n' + latest_ros_frame + b'\r\n')
        else:
            yield (b'--frame\r\n'
                   b'Content-Type: image/jpeg\r\n\r\n' + b'\r\n')

def generate_notebook_stream():
    global latest_notebook_frame
    notebook_cap = cv2.VideoCapture(0)
    while True:
        ret, frame = notebook_cap.read()
        if ret:
            ret, jpeg = cv2.imencode('.jpg', frame)
            if ret:
                latest_notebook_frame = jpeg.tobytes()
                yield (b'--frame\r\n'
                       b'Content-Type: image/jpeg\r\n\r\n' + latest_notebook_frame + b'\r\n')
        else:
            yield (b'--frame\r\n'
                   b'Content-Type: image/jpeg\r\n\r\n' + b'\r\n')

@app.route('/')
def index():
    if 'logged_in' not in session:
        return redirect(url_for('login'))
    return render_template('two_cam_index.html')  # Update template as needed

@app.route('/logout')
def logout():
    session.pop('logged_in', None)
    return redirect(url_for('login'))

@app.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']
        if username == 'admin' and password == 'password':
            session['logged_in'] = True
            return redirect(url_for('index'))
        else:
            return "Invalid credentials. Please try again."
    return render_template('login.html')

@app.route('/video_feed1')
def video_feed1():
    if 'logged_in' not in session:
        return redirect(url_for('login'))
    return Response(generate_ros_stream(), mimetype='multipart/x-mixed-replace; boundary=frame')

@app.route('/video_feed2')
def video_feed2():
    if 'logged_in' not in session:
        return redirect(url_for('login'))
    return Response(generate_notebook_stream(), mimetype='multipart/x-mixed-replace; boundary=frame')

def start_ros():
    try:
        rclpy.init()
        ros_node = ROSSubscriber()
        rclpy.spin(ros_node)
    except Exception as e:
        print(f"Error in ROS node: {e}")

def run_flask():
    try:
        app.run(host='0.0.0.0', port=5000, debug=True, use_reloader=False)
    except Exception as e:
        print(f"Error in Flask app: {e}")

def main(args=None):
    try:
        # Start ROS2 in a separate thread
        ros_thread = threading.Thread(target=start_ros)
        ros_thread.start()
        
        # Start Flask app in the main thread
        run_flask()
    except Exception as e:
        print(f"Error in main function: {e}")

if __name__ == '__main__':
    main()
